@@ -1,3 +1,6 @@
+// SPDX-FileCopyrightText: 2023 The Pion community <https://pion.ly>
+// SPDX-License-Identifier: MIT
+
 package srtp
 
 import (
@@ -8,21 +11,128 @@ import (
 	"time"
 
 	"github.com/pion/rtp"
-	"github.com/pion/transport/v2/packetio"
+	"github.com/pion/transport/v3/packetio"
 	"github.com/stretchr/testify/assert"
 )
 
 type noopConn struct{ closed chan struct{} }
 
-func newNoopConn() *noopConn                           { return &noopConn{closed: make(chan struct{})} }
-func (c *noopConn) Read(b []byte) (n int, err error)   { <-c.closed; return 0, io.EOF }
-func (c *noopConn) Write(b []byte) (n int, err error)  { return len(b), nil }
-func (c *noopConn) Close() error                       { close(c.closed); return nil }
-func (c *noopConn) LocalAddr() net.Addr                { return nil }
-func (c *noopConn) RemoteAddr() net.Addr               { return nil }
-func (c *noopConn) SetDeadline(t time.Time) error      { return nil }
-func (c *noopConn) SetReadDeadline(t time.Time) error  { return nil }
-func (c *noopConn) SetWriteDeadline(t time.Time) error { return nil }
+func newNoopConn() *noopConn { return &noopConn{closed: make(chan struct{})} }
+func (c *noopConn) Read([]byte) (n int, err error) {
+	<-c.closed
+
+	return 0, io.EOF
+}
+func (c *noopConn) Write(b []byte) (n int, err error) { return len(b), nil }
+func (c *noopConn) Close() error {
+	close(c.closed)
+
+	return nil
+}
+func (c *noopConn) LocalAddr() net.Addr              { return nil }
+func (c *noopConn) RemoteAddr() net.Addr             { return nil }
+func (c *noopConn) SetDeadline(time.Time) error      { return nil }
+func (c *noopConn) SetReadDeadline(time.Time) error  { return nil }
+func (c *noopConn) SetWriteDeadline(time.Time) error { return nil }
+
+func TestPeek(t *testing.T) {
+	firstBuffer := []byte{0xAA, 0xAA, 0xAA, 0xAA, 0xAA, 0xAA}
+	secondBuffer := []byte{0xBB, 0xBB, 0xBB}
+	thirdBuffer := []byte{0xCC, 0xCC, 0xCC}
+
+	buffer := packetio.NewBuffer()
+	stream := &ReadStreamSRTP{buffer: buffer}
+
+	t.Run("Short Peek", func(t *testing.T) {
+		_, err := buffer.Write(firstBuffer)
+		assert.NoError(t, err)
+
+		readBuff := make([]byte, 1)
+		_, err = stream.Peek(readBuff)
+		assert.Error(t, err, io.ErrShortBuffer)
+	})
+
+	t.Run("Short Read", func(t *testing.T) {
+		_, err := buffer.Write(firstBuffer)
+		assert.NoError(t, err)
+
+		readBuff := make([]byte, 6)
+		n, err := stream.Peek(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 6)
+		assert.Equal(t, readBuff, firstBuffer)
+
+		n, err = stream.Read([]byte{})
+		assert.Error(t, err, io.ErrShortBuffer)
+		assert.Equal(t, n, 0)
+		assert.Equal(t, readBuff, firstBuffer)
+
+		n, err = stream.Read(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 6)
+		assert.Equal(t, readBuff, firstBuffer)
+	})
+
+	t.Run("Single Peek", func(t *testing.T) {
+		_, err := buffer.Write(firstBuffer)
+		assert.NoError(t, err)
+
+		readBuff := make([]byte, 6)
+
+		n, err := stream.Peek(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 6)
+		assert.Equal(t, readBuff, firstBuffer)
+
+		n, err = stream.Read(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 6)
+		assert.Equal(t, readBuff, firstBuffer)
+	})
+
+	t.Run("Multi Peek", func(t *testing.T) {
+		_, err := buffer.Write(firstBuffer)
+		assert.NoError(t, err)
+
+		_, err = buffer.Write(secondBuffer)
+		assert.NoError(t, err)
+
+		_, err = buffer.Write(thirdBuffer)
+		assert.NoError(t, err)
+
+		readBuff := make([]byte, 6)
+
+		n, err := stream.Peek(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 6)
+		assert.Equal(t, readBuff[:n], firstBuffer)
+
+		n, err = stream.Peek(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 3)
+		assert.Equal(t, readBuff[:n], secondBuffer)
+
+		n, err = stream.Peek(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 3)
+		assert.Equal(t, readBuff[:n], thirdBuffer)
+
+		n, err = stream.Read(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 6)
+		assert.Equal(t, readBuff[:n], firstBuffer)
+
+		n, err = stream.Read(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 3)
+		assert.Equal(t, readBuff[:n], secondBuffer)
+
+		n, err = stream.Read(readBuff)
+		assert.NoError(t, err)
+		assert.Equal(t, n, 3)
+		assert.Equal(t, readBuff[:n], thirdBuffer)
+	})
+}
 
 func TestBufferFactory(t *testing.T) {
 	wg := sync.WaitGroup{}
@@ -30,6 +140,7 @@ func TestBufferFactory(t *testing.T) {
 	conn := newNoopConn()
 	bf := func(_ packetio.BufferPacketType, _ uint32) io.ReadWriteCloser {
 		wg.Done()
+
 		return packetio.NewBuffer()
 	}
 	rtpSession, err := NewSessionSRTP(conn, &Config{
@@ -62,13 +173,15 @@ func TestBufferFactory(t *testing.T) {
 }
 
 func benchmarkWrite(b *testing.B, profile ProtectionProfile, size int) {
+	b.Helper()
+
 	conn := newNoopConn()
 
-	keyLen, err := profile.keyLen()
+	keyLen, err := profile.KeyLen()
 	if err != nil {
 		b.Fatal(err)
 	}
-	saltLen, err := profile.saltLen()
+	saltLen, err := profile.SaltLen()
 	if err != nil {
 		b.Fatal(err)
 	}
@@ -140,15 +253,17 @@ func BenchmarkWrite(b *testing.B) {
 }
 
 func benchmarkWriteRTP(b *testing.B, profile ProtectionProfile, size int) {
+	b.Helper()
+
 	conn := &noopConn{
 		closed: make(chan struct{}),
 	}
 
-	keyLen, err := profile.keyLen()
+	keyLen, err := profile.KeyLen()
 	if err != nil {
 		b.Fatal(err)
 	}
-	saltLen, err := profile.saltLen()
+	saltLen, err := profile.SaltLen()
 	if err != nil {
 		b.Fatal(err)
 	}
